# -*- coding: utf-8 -*-
import os
from gestionatr.output.messages.base import Cabecera
from gestionatr.output.messages import sw_c1 as c1
from gestionatr.output.messages import sw_c2 as c2
from . import unittest

_ROOT = os.path.abspath(os.path.dirname(__file__))


def get_data(path):
    filename = isinstance(path, (list, tuple)) and path[0] or path
    return os.path.join(_ROOT, 'data', filename)


def get_header(process='C1', step='01', code='201607211259', date='2016-07-21T12:59:47'):
    header = Cabecera()
    vals = {
        'codigo_del_proceso': process,
        'codigo_del_paso': step,
        'codigo_de_solicitud': code,
        'secuencial_de_solicitud': '01',
        'cups': 'ES1234000000000001JN0F',
        'codigo_ree_empresa_emisora': '1234',
        'codigo_ree_empresa_destino': '4321',
        'fecha': date,
    }
    header.feed(vals)
    return header


def get_cliente(dir=False, tipo_dir=None):
    # Cliente
    cliente = c2.Cliente()

    # IdCliente
    id_cliente = c2.IdCliente()
    id_cliente_fields = {
        'tipo_identificador': 'NI',
        'identificador': 'B36385870',
        'tipo_persona': 'J',
    }
    id_cliente.feed(id_cliente_fields)

    # Nombre
    nombre = c2.Nombre()
    nombre_fields = {
        'nombre_de_pila': '',
        'primer_apellido': '',
        'segundo_apellido': '',
        'razon_social': u'ACC Y COMP DE COCINA MILLAN Y MUÑOZ',
    }
    nombre.feed(nombre_fields)

    # Telefono
    telefono1 = c2.Telefono()
    telefono_fields = {
        'prefijo_pais': '36',
        'numero': '666777666',
    }
    telefono1.feed(telefono_fields)
    telefono2 = c2.Telefono()
    telefono_fields = {
        'prefijo_pais': '37',
        'numero': '666777777',
    }
    telefono2.feed(telefono_fields)
    telefono3 = c2.Telefono()
    telefono_fields = {
        'prefijo_pais': '38',
        'numero': '666777888',
    }
    telefono3.feed(telefono_fields)
    telefonos = [telefono1, telefono2, telefono3]

    indicador_tipo_direccion = None
    if tipo_dir:
        indicador_tipo_direccion = tipo_dir
    direccion = None
    if dir == "correo":
        # Direccion
        direccion = c2.Direccion()
        direccion_fields = {
            'pais': 'España',
            'provincia': '17',
            'municipio': '17079',
            'poblacion': '17079',
            'apartado_de_correos': "A1234",
            'cod_postal': '17001',
        }
        direccion.feed(direccion_fields)
    elif dir:
        via = c2.Via()
        via.feed({
            'tipo_via': 'PZ',
            'calle': 'MELA MUTERMILCH',
            'numero_finca': '2',
            'piso': '001',
            'puerta': '001',
            'tipo_aclarador_finca': 'BI',
            'aclarador_finca': 'Bloque de Pisos',
        })
        # Direccion
        direccion = c2.Direccion()
        direccion_fields = {
            'pais': 'España',
            'provincia': '17',
            'municipio': '17079',
            'poblacion': '17079',
            'via': via,
            'cod_postal': '17001',
        }
        direccion.feed(direccion_fields)

    cliente_fields = {
        'id_cliente': id_cliente,
        'nombre': nombre,
        'telefonos': telefonos,
        'correo_electronico': 'email@host',
        'indicador_tipo_direccion': indicador_tipo_direccion,
        'direccion': direccion
    }
    cliente.feed(cliente_fields)
    return cliente


def get_contacto(email=True):
    # Contacto
    contacto = c2.Contacto()

    # Telefono
    telefono = c2.Telefono()
    telefono_fields = {
        'prefijo_pais': '34',
        'numero': '666777888',
    }
    telefono.feed(telefono_fields)
    telefono2 = c2.Telefono()
    telefono_fields = {
        'prefijo_pais': '34',
        'numero': '666777999',
    }
    telefono2.feed(telefono_fields)
    telefonos = [telefono, telefono2]

    correo = None
    if email:
        correo = 'email@host'
    contacto_fields = {
        'persona_de_contacto': 'Nombre Inventado',
        'telefonos': telefonos,
        'correo_electronico': correo,
    }
    contacto.feed(contacto_fields)
    return contacto


def get_medida():
    # Medida
    medida = c2.Medida()

    # ModelosAparato
    md1 = c2.ModeloAparato()
    modelo_aparato_fields = {
        'tipo_aparato': 'CG',
        'marca_aparato': '132',
        'modelo_marca': '011',
    }
    md1.feed(modelo_aparato_fields)
    md2 = c2.ModeloAparato()
    modelo_aparato_fields = {
        'tipo_aparato': 'CG',
        'marca_aparato': '132',
        'modelo_marca': '012',
    }
    md2.feed(modelo_aparato_fields)

    modelos_aparato = c2.ModelosAparato()
    modelos_aparato_fields = {
        'modelo_aparato_list': [md1, md2],
    }
    modelos_aparato.feed(modelos_aparato_fields)

    medida_fields = {
        'propiedad_equipo': 'C',
        'tipo_equipo_medida': 'L00',
        'modelos_aparato': modelos_aparato,
    }
    medida.feed(medida_fields)
    return medida


def assertXmlEqual(got, want):
    from lxml.doctestcompare import LXMLOutputChecker
    from doctest import Example

    checker = LXMLOutputChecker()
    if checker.check_output(want, got, 0):
        return
    message = checker.output_difference(Example("", want), got, 0)
    raise AssertionError(message)

unittest.TestCase.assertXmlEqual = assertXmlEqual
unittest.TestCase.__str__ = unittest.TestCase.id


# -*- coding: utf-8 -*-
from gestionatr.output.messages.sw_c2 import *


# Paso 01
class MensajeBajaSuspension(XmlModel):

    _sort_order = ('mensaje', 'cabecera', 'baja_suspension')

    def __init__(self):
        self.mensaje = XmlField('MensajeBajaSuspension',
                                attributes={'xmlns': 'http://localhost/elegibilidad'})
        self.cabecera = Cabecera()
        self.baja_suspension = BajaSuspension()
        super(MensajeBajaSuspension, self).__init__('MensajeBajaSuspension', 'mensaje')


class BajaSuspension(XmlModel):

    _sort_order = ('baja_suspension', 'datos_solicitud', 'cliente', 'contacto', 'iban', 'comentarios', 'registros_documento')

    def __init__(self):
        self.baja_suspension = XmlField('BajaSuspension')
        self.datos_solicitud = DatosSolicitud()
        self.cliente = Cliente()
        self.contacto = Contacto()
        self.iban = XmlField('IBAN')
        self.comentarios = XmlField('Comentarios')
        self.registros_documento = RegistrosDocumento()
        super(BajaSuspension, self).__init__('BajaSuspension', 'baja_suspension')


class DatosSolicitud(DatosSolicitud):

    _sort_order = ('datos_solicitud', 'tipo_modificacion', 'tipo_solicitud_administrativa', 'cnae', 'ind_activacion', 'fecha_prevista_accion', 'motivo', 'contratacion_incondicional_ps')

    def __init__(self):
        super(DatosSolicitud, self).__init__()
        self.motivo = XmlField('Motivo')


# Paso 02
class MensajeAceptacionBajaSuspension(XmlModel):

    _sort_order = ('mensaje', 'cabecera', 'aceptacion_baja_suspension')

    def __init__(self):
        self.mensaje = XmlField('MensajeAceptacionBajaSuspension',
                                attributes={'xmlns': 'http://localhost/elegibilidad'})
        self.cabecera = Cabecera()
        self.aceptacion_baja_suspension = AceptacionBajaSuspension()
        super(MensajeAceptacionBajaSuspension, self).__init__('MensajeAceptacionBajaSuspension', 'mensaje')


class AceptacionBajaSuspension(XmlModel):

    _sort_order = ('aceptacion_baja_suspension', 'datos_aceptacion')

    def __init__(self):
        self.aceptacion_baja_suspension = XmlField('AceptacionBajaSuspension')
        self.datos_aceptacion = DatosAceptacion()
        super(AceptacionBajaSuspension, self).__init__('AceptacionBajaSuspension', 'aceptacion_baja_suspension')


class DatosAceptacion(DatosAceptacion):

    _sort_order = ('datos_aceptacion', 'fecha_aceptacion', 'potencia_actual', 'actuacion_campo', 'fecha_ultima_lectura_firme', 'tipo_activacion_prevista', 'fecha_activacion_prevista')

    def __init__(self):
        super(DatosAceptacion, self).__init__()
        self.tipo_activacion_prevista = XmlField('TipoActivacionPrevista')
        self.fecha_activacion_prevista = XmlField('FechaActivacionPrevista')


# Paso 03 -> C1 MensajeAnulacionSolicitud

# Paso 04
class MensajeAceptacionAnulacionBaja(XmlModel):

    _sort_order = ('mensaje', 'cabecera', 'aceptacion_anulacion')

    def __init__(self):
        self.mensaje = XmlField('MensajeAceptacionAnulacionBaja',
                                attributes={'xmlns': 'http://localhost/elegibilidad'})
        self.cabecera = Cabecera()
        self.aceptacion_anulacion = AceptacionAnulacion()
        super(MensajeAceptacionAnulacionBaja, self).__init__('MensajeAceptacionAnulacionBaja', 'mensaje')


class AceptacionAnulacion(AceptacionAnulacion):

    _sort_order = ('aceptacion_anulacion', 'fecha_aceptacion', 'hora_aceptacion')

    def __init__(self):
        super(AceptacionAnulacion, self).__init__()
        self.hora_aceptacion = XmlField('HoraAceptacion')


# Paso 05
class MensajeActivacionBajaSuspension(XmlModel):

    _sort_order = ('mensaje', 'cabecera', 'activacion_baja')

    def __init__(self):
        self.mensaje = XmlField('MensajeActivacionBajaSuspension',
                                attributes={'xmlns': 'http://localhost/elegibilidad'})
        self.cabecera = Cabecera()
        self.activacion_baja = ActivacionBaja()
        super(MensajeActivacionBajaSuspension, self).__init__('MensajeActivacionBajaSuspension', 'mensaje')


class ActivacionBaja(XmlModel):

    _sort_order = ('activacion_baja', 'datos_activacion_baja', 'contrato', 'puntos_de_medida')

    def __init__(self):
        self.activacion_baja = XmlField('ActivacionBaja')
        self.datos_activacion_baja = DatosActivacionBaja()
        self.contrato = Contrato()
        self.puntos_de_medida = PuntosDeMedida()
        super(ActivacionBaja, self).__init__('ActivacionBaja', 'activacion_baja')


class DatosActivacionBaja(XmlModel):

    _sort_order = ('datos_activacion_baja', 'fecha_activacion')

    def __init__(self):
        self.datos_activacion_baja = XmlField('DatosActivacionBaja')
        self.fecha_activacion = XmlField('FechaActivacion')
        super(DatosActivacionBaja, self).__init__('DatosActivacionBaja', 'datos_activacion_baja')

# Paso 06 -> C2 MensajeIncidenciasATRDistribuidor
# Paso 07 -> C1 MensajeRechazo



# -*- coding: utf-8 -*-
from message import Message
from gestionatr.utils import get_rec_attr
from Deadlines import ProcessDeadline


class Q1(Message, ProcessDeadline):
    """Clase que implementa Q1."""

    @property
    def cod_pm(self):
        tree = '{0}.CodPM'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data:
            return data.text
        else:
            return False

    @property
    def modelos_aparato(self):
        data = []
        obj = get_rec_attr(self.obj, self._header, False)
        for i in obj.ModeloAparato:
            data.append(ModeloAparato(i))
        return data


class ModeloAparato(object):

    def __init__(self, data):
        self.modelo_aparato = data

    @property
    def tipo_aparato(self):
        data = ''
        try:
            data = self.modelo_aparato.TipoAparato.text
        except AttributeError:
            pass
        return data

    @property
    def marca_aparato(self):
        data = ''
        try:
            data = self.modelo_aparato.MarcaAparato.text
        except AttributeError:
            pass
        return data

    @property
    def numero_serie(self):
        data = ''
        try:
            data = self.modelo_aparato.NumeroSerie.text
        except AttributeError:
            pass
        return data

    @property
    def tipo_dhedm(self):
        data = ''
        try:
            data = self.modelo_aparato.TipoDHEdM.text
        except AttributeError:
            pass
        return data

    @property
    def integradores(self):
        data = []
        for i in self.modelo_aparato.Integrador:
            data.append(Integrador(i))
        return data


class Integrador(object):

    def __init__(self, data):
        self.integrador = data

    @property
    def magnitud(self):
        data = ''
        try:
            data = self.integrador.Magnitud.text
        except AttributeError:
            pass
        return data

    @property
    def codigo_periodo(self):
        data = ''
        try:
            data = self.integrador.CodigoPeriodo.text
        except AttributeError:
            pass
        return data

    @property
    def constante_multiplicadora(self):
        data = ''
        try:
            data = self.integrador.ConstanteMultiplicadora.text
        except AttributeError:
            pass
        return data

    @property
    def numero_ruedas_enteras(self):
        data = ''
        try:
            data = self.integrador.NumeroRuedasEnteras.text
        except AttributeError:
            pass
        return data

    @property
    def numero_ruedas_decimales(self):
        data = ''
        try:
            data = self.integrador.NumeroRuedasDecimales.text
        except AttributeError:
            pass
        return data

    @property
    def consumo_calculado(self):
        data = ''
        try:
            data = self.integrador.ConsumoCalculado.text
        except AttributeError:
            pass
        return data

    @property
    def lectura_desde(self):
        data = ''
        try:
            data = LecturaInfo(self.integrador.LecturaDesde)
        except AttributeError:
            pass
        return data

    @property
    def lectura_hasta(self):
        data = ''
        try:
            data = LecturaInfo(self.integrador.LecturaHasta)
        except AttributeError:
            pass
        return data

    @property
    def ajuste(self):
        data = ''
        try:
            data = Ajuste(self.integrador.Ajuste)
        except AttributeError:
            pass
        return data

    @property
    def anomalia(self):
        data = ''
        try:
            data = Anomalia(self.integrador.Anomalia)
        except AttributeError:
            pass
        return data

    @property
    def fecha_hora_maximetro(self):
        data = False
        try:
            data = self.integrador.FechaHoraMaximetro.text
        except AttributeError:
            pass
        return data


class LecturaInfo(object):

    def __init__(self, data):
        self.lect = data

    @property
    def fecha(self):
        data = False
        try:
            data = self.lect.Fecha.text
        except AttributeError:
            pass
        return data

    @property
    def procedencia(self):
        data = ''
        try:
            data = self.lect.Procedencia.text
        except AttributeError:
            pass
        return data

    @property
    def lectura(self):
        data = ''
        try:
            data = self.lect.Lectura.text
        except AttributeError:
            pass
        return data


class Ajuste(object):

    def __init__(self, data):
        self.ajuste = data

    @property
    def codigo_motivo_ajuste(self):
        data = ''
        try:
            data = self.ajuste.CodigoMotivoAjuste.text
        except AttributeError:
            pass
        return data

    @property
    def ajuste_por_integrador(self):
        data = ''
        try:
            data = self.ajuste.AjustePorIntegrador.text
        except AttributeError:
            pass
        return data

    @property
    def comentarios(self):
        data = ''
        try:
            data = self.ajuste.Comentarios.text
        except AttributeError:
            pass
        return data


class Anomalia(object):

    def __init__(self, data):
        self.anomalia = data

    @property
    def tipo_anomalia(self):
        data = ''
        try:
            data = self.anomalia.TipoAnomalia.text
        except AttributeError:
            pass
        return data

    @property
    def comentarios(self):
        data = ''
        try:
            data = self.anomalia.Comentarios.text
        except AttributeError:
            pass
        return data


file = open("info")
out = open("out.txt", 'w')

init_i = "\nclass {0}(XmlModel):\n\n" \
         "    _sort_order = ({1})\n\n" \
         "    def __init__(self):\n" \
         "        self.{2} = XmlField('{0}')\n"

satr_line = "        self.{0} = XmlField('{1}')\n"
aatr_line = "        self.{0} = {1}()\n"
latr_line = "        self.{0} = []\n"
final_line = "        super({0}, self).__init__('{1}', '{2}')\n\n\n"


classname = file.readline()[1:-2]
atr_names_list = [classname]
names_list = [classname]

for line in file.readlines():
    aname = line[1:-2]
    atr_names_list.append(aname)
    names_list.append(line[1:-2])

out.write(init_i.format(classname, str(atr_names_list)[1:-1], atr_names_list[0]))

for i in range(1, len(atr_names_list)):
    out.write(satr_line.format(atr_names_list[i], names_list[i]))

out.write(final_line.format(classname, classname, atr_names_list[0]))



file = open("info")
out = open("out.txt", 'w')

method = "@property\n" \
         "def {0}(self):\n" \
         "    tree = {2}\n" \
         "    data = get_rec_attr(self.{1}, tree, False)\n" \
         "    if data is not None and data is not False:\n" \
         "        return data.text\n" \
         "    else:\n" \
         "        return False\n\n"

method_class = "@property\n" \
         "def {0}(self):\n" \
         "    tree = {2}\n" \
         "    data = get_rec_attr(self.{1}, tree, False)\n" \
         "    if data is not None and data is not False:\n" \
         "        return {3}(data)\n" \
         "    else:\n" \
         "        return False\n\n"

method_list = "@property\n" \
               "def {0}(self):\n" \
               "    data = []\n" \
               "    obj = get_rec_attr(self.obj, '{2}', False)\n" \
               "    if obj is not None and obj is not False:\n" \
               "        for d in obj.{0}:\n" \
               "            data.append({1}(d))\n" \
               "    return data\n\n"

method_list2 = "@property\n" \
              "def {0}(self):\n" \
              "    data = []\n" \
              "    obj = get_rec_attr(self.obj, '{3}', False)\n" \
              "    if (hasattr(obj, '{3}') and\n" \
              "            hasattr(obj.{3}, '{1}')):\n" \
              "        for d in obj.{3}.{1}:\n" \
              "            data.append({2}(d))\n" \
              "    return data\n\n"

obj = "obj"
use_alt_tree = True

from A1_44 import A1_44 as check_repeated

for line in file.readlines():
    use2, use3, use4 = False, False, False
    if line[0] == '*':
        line = line[1:]
        use2 = True
    elif line[0] == '+':
        if line[1] == '+':
            line = line[2:]
            use4 = True
        else:
            line = line[1:]
            use3 = True

    line = line[1:-2]
    cname = line.title()
    mname = line[0].lower()
    for c in line[1:]:
        if c.isupper():
            mname += "_{0}".format(c.lower())
        else:
            mname += c

    tree_str = "'{{0}}.{0}'.format(self._header)".format(line)
    if use_alt_tree:
        tree_str = "'{0}'".format(line)

    if use2:
        if check_repeated:
            if not getattr(check_repeated, mname, False):
                out.write(method_class.format(
                    mname,
                    obj,
                    tree_str,
                    cname,
                ))
        else:
            out.write(method_class.format(
                mname,
                obj,
                tree_str,
                cname,
            ))
    elif use3:
        if check_repeated:
            if not getattr(check_repeated, mname, False):
                out.write(method_list.format(
                    mname,
                    cname,
                    line
                ))
        else:
            out.write(method_list.format(
                mname,
                cname,
                line
            ))
    elif use4:
        mname_ind = mname[:-4]
        if mname_ind.endswith('es'):
            mname_ind = mname_ind[:-2]
        elif mname_ind.endswith('s'):
            mname_ind = mname_ind[:-1]
        cname = mname_ind.title()
        if check_repeated:
            if not getattr(check_repeated, mname, False):
                out.write(method_list2.format(
                    mname,
                    mname_ind,
                    cname,
                    line
                ))
        else:
            out.write(method_list2.format(
                mname,
                mname_ind,
                cname,
                line
            ))
    else:
        if check_repeated:
            if not getattr(check_repeated, mname, False):
                out.write(method.format(
                    mname,
                    obj,
                    tree_str,
                ))
        else:
            out.write(method.format(
                mname,
                obj,
                tree_str,
            ))



# -*- coding: utf-8 -*-
from message import Message, except_f1
from C1 import C1
from C2 import C2
from A3 import A3
from B1 import B1
from M1 import M1
from D1 import D1
from W1 import W1
from Q1 import Q1
from R1 import R1
from F1 import F1
from message_gas import MessageGas
from A1_41 import A1_41
from B70 import B7031, B7032, B70
from A1_02 import A1_02
from A1_05 import A1_05
from A1_44 import A1_44
from A1_03 import A1_03
from A1_04 import A1_04
from A1_48 import A1_48
from A1_46 import A1_46
